@@ -109,6 +109,18 @@ g_usb_device_get_property (GObject    *object,
 	}
 }
 
+static void
+set_libusb_device (GUsbDevice           *device,
+		   struct libusb_device *dev)
+{
+	GUsbDevicePrivate *priv = device->priv;
+
+	g_clear_pointer (&priv->device, libusb_unref_device);
+
+	if (dev != NULL)
+		priv->device = libusb_ref_device (dev);
+}
+
 static void
 g_usb_device_set_property (GObject      *object,
 			   guint	 prop_id,
@@ -120,7 +132,7 @@ g_usb_device_set_property (GObject      *object,
 
 	switch (prop_id) {
 	case PROP_LIBUSB_DEVICE:
-		priv->device = g_value_get_pointer (value);
+		set_libusb_device (device, g_value_get_pointer (value));
 		break;
 	case PROP_CONTEXT:
 		priv->context = g_value_dup_object (value);
@@ -146,8 +158,6 @@ g_usb_device_constructed (GObject *object)
 	if (!priv->device)
 		g_error("constructed without a libusb_device");
 
-	libusb_ref_device(priv->device);
-
 	rc = libusb_get_device_descriptor (priv->device, &priv->desc);
 	if (rc != LIBUSB_SUCCESS)
 		g_warning ("Failed to get USB descriptor for device: %s",
@@ -202,6 +212,24 @@ g_usb_device_init (GUsbDevice *device)
 	device->priv = g_usb_device_get_instance_private (device);
 }
 
+/* not defined in FreeBSD */
+#ifndef HAVE_LIBUSB_GET_PARENT
+static libusb_device *
+libusb_get_parent (libusb_device *dev)
+{
+	return NULL;
+}
+#endif
+
+/* not defined in DragonFlyBSD */
+#ifndef HAVE_LIBUSB_GET_PORT_NUMBER
+static guint8
+libusb_get_port_number (libusb_device *dev)
+{
+	return 0xff;
+}
+#endif
+
 static void
 g_usb_device_build_parent_port_number (GString *str, libusb_device *dev)
 {
@@ -241,8 +269,6 @@ g_usb_device_initable_init (GInitable     *initable,
 		return FALSE;
 	}
 
-	libusb_ref_device (priv->device);
-
 	rc = libusb_get_device_descriptor (priv->device, &priv->desc);
 	if (rc != LIBUSB_SUCCESS) {
 		g_set_error (error, G_USB_DEVICE_ERROR, G_USB_DEVICE_ERROR_INTERNAL,
